package com.galaxis.manatee.controller;

import com.galaxis.manatee.dao.ChuanyunUserCompanyDao;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunUserCompanyDO;
import com.galaxis.manatee.service.*;
import com.galaxis.manatee.task.ChuanyunBasicDataScheduledTask;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/10/27 11:39 上午
 */
@Slf4j
@RestController
public class TestController {

    @Autowired
    private ChuanyunUserCompanyDao chuanyunUserCompanyDao;

    private final LogUpdateService logUpdateService;
    private final LogStandardService logStandardService;
    private final LogCheckService logCheckService;
    private final LogService logService;
    private final LogDownloadService logDownloadService;

    private final ProjectMemberService projectMemberService;

    private final ChuanyunBasicDataScheduledTask chuanyunBasicDataScheduledTask;

    @Autowired
    private DingTalkScheduledTask dingTalkScheduledTask;


    public TestController(LogUpdateService logUpdateService, LogStandardService logStandardService, LogCheckService logCheckService, LogService logService, LogDownloadService logDownloadService, ProjectMemberService projectMemberService, ChuanyunBasicDataScheduledTask chuanyunBasicDataScheduledTask) {
        this.logUpdateService = logUpdateService;
        this.logStandardService = logStandardService;
        this.logCheckService = logCheckService;
        this.logService = logService;
        this.logDownloadService = logDownloadService;
        this.projectMemberService = projectMemberService;
        this.chuanyunBasicDataScheduledTask = chuanyunBasicDataScheduledTask;
    }

    @GetMapping("/test/helloGet")
    public String testGet() {
        return "hello";
    }

    /**
     * 标准化所有周工时
     */
    @GetMapping("/test/api/standardAllWeekMemberHour")
    public void standardAllWeekMemberHour() {
        logService.standardAllWeekMemberHour();
    }

    /**
     * 标准化所有月工时
     */
    @GetMapping("/test/api/standardAllSelfCost")
    public void standardAllSelfCost() {
        logService.standardAllSelfCost();
    }

    /**
     * 根据用户Id标准化日工时
     *
     * @param userId 用户Id
     */
    @GetMapping("/test/api/standardSelfWorkHourByUserId")
    public void standardSelfWorkHourByUserId(@RequestParam("userId") String userId) {
        logStandardService.standardSelfWorkHourByUserId(userId);
    }

    /**
     * 根据用户Id标准化周工时
     *
     * @param userId 用户Id
     */
    @GetMapping("/test/api/standardWeekMemberHourByUserId")
    public void standardWeekMemberHourByUserId(@RequestParam("userId") String userId) {
        logStandardService.standardWeekMemberHourByUserId(userId);
    }

    /**
     * 根据用户Id标准化月工时
     *
     * @param userId 用户Id
     */
    @GetMapping("/test/api/standardSelfCostByUserId")
    public void standardSelfCostByUserId(@RequestParam("userId") String userId) {
        logStandardService.standardSelfCostByUserId(userId);
    }

    /**
     * 更新所有每天工时
     */
    @GetMapping("/test/api/updateAllChuanyunSelfWorkHour")
    public void updateAllChuanyunSelfWorkHour() {
        logService.updateAllChuanyunSelfWorkHour();
    }

    /**
     * 更新所有每周工时
     */
    @GetMapping("/test/api/updateAllChuanyunWeekMemberHour")
    public void updateAllChuanyunWeekMemberHour() {
        logService.updateAllChuanyunWeekMemberHour();
    }

    /**
     * 更新所有每月工时
     */
    @GetMapping("/test/api/updateAllChuanyunSelfCost")
    public void updateAllChuanyunSelfCost() {
        logService.updateAllChuanyunSelfCost();
    }

    /**
     * 根据用户Id更新每天工时方法
     *
     * @param userId 用户Id
     */
    @GetMapping("/test/api/updateChuanyunSelfWorkHourListByUserId")
    public void updateChuanyunSelfWorkHourListByUserId(@RequestParam("userId") String userId) {
        logUpdateService.updateChuanyunSelfWorkHourListByUserId(userId);
    }

    /**
     * @param userId 用户Id
     */
    @GetMapping("/test/api/updateChuanyunWeekMemberHourListByUserId")
    public void updateChuanyunWeekMemberHourListByUserId(@RequestParam("userId") String userId) {
        logUpdateService.updateChuanyunWeekMemberHourListByUserId(userId);
    }

    /**
     * @param userId 用户Id
     */
    @GetMapping("/test/api/updateChuanyunSelfCostListByUserId")
    public void updateChuanyunSelfCostListByUserId(@RequestParam("userId") String userId) {
        logUpdateService.updateChuanyunSelfCostListByUserId(userId);
    }

    /**
     * 根据用户Id核查工时信息
     *
     * @param userId 用户Id
     */
    @GetMapping("/test/api/checkWorkHourByUserId")
    public void checkWorkHourByUserId(@RequestParam("userId") String userId) {
        long start = System.currentTimeMillis();
        try {
            //核对原始数据和标准化每日工时
            logCheckService.checkDayWorkHour(userId);
            logCheckService.checkWeekWorkHour(userId);
            logCheckService.checkMonthWorkHour(userId);
        } catch (Exception e) {
            log.error(userId);
            log.error(e.getMessage());
        }
        log.info("修改" + userId + "工时信息耗时" + (System.currentTimeMillis() - start));

    }

    /**
     * 核查工时信息
     *
     * @param
     */
    @GetMapping("/test/api/checkWorkHour")
    public void checkWorkHour() {
        List<String> allUserIdList = chuanyunUserCompanyDao.findAll().stream().map(ChuanyunUserCompanyDO::getUserId).collect(Collectors.toList());
        List<String> disUserIdList = allUserIdList.stream().distinct().collect(Collectors.toList());
        logCheckService.checkDayWorkHourTimeAfter(disUserIdList, 90);
        logCheckService.checkWeekWorkHour(disUserIdList);
        logCheckService.checkMonthWorkHour(disUserIdList);

    }

    /**
     * 下载工时日志
     *
     * @param
     */
    @GetMapping("/test/api/pullChuanyunLog")
    public void pullChuanyunLog() {
        logDownloadService.saveChuanyunLogDO(false);
    }
    /**
     * 下载工时日志
     *
     * @param
     */
    @GetMapping("/test/api/pullAllChuanyunLog")
    public void pullAllChuanyunLog() {
        logDownloadService.saveAllLog();
    }

    /**
     * 手动更新项目组成员
     */
    @GetMapping("/test/api/manual/updateProjectMember")
    public void updateProjectMember() {
        try {
            projectMemberService.updateManagerMember("bfc36277-b201-4a80-b61c-e632bc664d82");
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    /**
     * 手动更新项目组成员
     */
    @GetMapping("/test/api/manual/updateSaleProjectMember")
    public void updateSaleProjectMember() {
        try {
            projectMemberService.updateSaleMember("bfc36277-b201-4a80-b61c-e632bc664d82");
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    /**
     * 手动更新项目组成员
     */
    @GetMapping("/test/api/manual/getGroupProjectForSynchronizationManually")
    public void getGroupProjectForSynchronizationManually() {

        chuanyunBasicDataScheduledTask.getGroupProjectForSynchronizationManually();

    }

    /**
     * 上传请假信息
     */
    @GetMapping("/test/pushAskForLeave")
    public void pushAskForLeave() {

        chuanyunBasicDataScheduledTask.pushAskForLeave();

    }

    /**
     * 上传请假信息
     */
    @GetMapping("/test/pullAskForLeave")
    public void pullAskForLeave() {
        dingTalkScheduledTask.manuallyAskForLeave();

    }

}


