package com.galaxis.manatee.task;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.galaxis.manatee.capsule.dto.PageDTO;
import com.galaxis.manatee.dao.*;
import com.galaxis.manatee.entity.chuanyun.data.object.*;
import com.galaxis.manatee.entity.chuanyun.dto.*;
import com.galaxis.manatee.exception.BigSizeException;
import com.galaxis.manatee.manager.ChuanYunManager;
import com.galaxis.manatee.manager.ClawFeign;
import com.galaxis.manatee.util.ChuanyunLocalDateTimeDeserializer;
import com.galaxis.manatee.util.ChuanyunLocalDateTimeSerializer;
import com.github.pagehelper.PageInfo;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.persistence.CascadeType;
import javax.persistence.FetchType;
import javax.persistence.OneToMany;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

@Service
@Slf4j
public class U9InfoTask {

    private final Integer pageSize = 20;
    private final ChuanYunManager chuanYunManager;
    private final ClawFeign clawFeign;
    private final ChuanyunBudgetStatisticsDao chuanyunBudgetStatisticsDao;
    private final ChuanyunCompanyDao chuanyunCompanyDao;
    private final ChuanyunClientDao chuanyunClientDao;
    private final ChuanyunUserDao chuanyunUserDao;
    private final ChuanyunOutDepartmentDao chuanyunOutDepartmentDao;
    private final ChuanyunSupplierDao chuanyunSupplierDao;
    private ChuanyunMonthLaborCostWgDao chuanyunMonthLaborCostWgDao;
    private ChuanyunMonthLaborCostNewDao chuanyunMonthLaborCostNewDao;

    @Autowired
    public void setChuanyunMonthLaborCostWgDao(ChuanyunMonthLaborCostWgDao chuanyunMonthLaborCostWgDao) {
        this.chuanyunMonthLaborCostWgDao = chuanyunMonthLaborCostWgDao;
    }

    @Autowired
    public void setChuanyunMonthLaborCostNewDao(ChuanyunMonthLaborCostNewDao chuanyunMonthLaborCostNewDao) {
        this.chuanyunMonthLaborCostNewDao = chuanyunMonthLaborCostNewDao;
    }

    public U9InfoTask(ChuanYunManager chuanYunManager, ClawFeign clawFeign, ChuanyunBudgetStatisticsDao chuanyunBudgetStatisticsDao, ChuanyunCompanyDao chuanyunCompanyDao, ChuanyunClientDao chuanyunClientDao, ChuanyunUserDao chuanyunUserDao, ChuanyunOutDepartmentDao chuanyunOutDepartmentDao, ChuanyunSupplierDao chuanyunSupplierDao) {
        this.chuanYunManager = chuanYunManager;
        this.clawFeign = clawFeign;
        this.chuanyunBudgetStatisticsDao = chuanyunBudgetStatisticsDao;
        this.chuanyunCompanyDao = chuanyunCompanyDao;
        this.chuanyunClientDao = chuanyunClientDao;
        this.chuanyunUserDao = chuanyunUserDao;
        this.chuanyunOutDepartmentDao = chuanyunOutDepartmentDao;
        this.chuanyunSupplierDao = chuanyunSupplierDao;
    }

    @Scheduled(fixedDelay = 3600000L, initialDelay = 3600000L)
    private void getDataFromU9() {
//        savePurCost();
//        saveCashFlow();
        saveBudgetStatistics();
        savePurContract();
        saveSalesContract();
        saveSalesCollection();
        saveSupplier();
    }

    @Scheduled(cron = "0 0 3 * * *")
    private void getWorkGroupCost() {
        saveLaborCost();
        saveDbLaborCostWg();
        saveSubLaborCost();
    }

    @Scheduled(cron = "0 0 0 15 * ? *")
    private void checkLaborCost() {
        saveDbLaborCost();
    }

    /**
     * 更新项目物料成本
     */
    private void savePurCost() {
        log.info("开始更新项目物料成本");
        var objectMapper = new ObjectMapper();
        var page = 1;
        var size = 20;
        var flag = true;
        while (flag) {
            Map<String, String> map = new HashMap<>(1);
            map.put("page", String.valueOf(page));
            map.put("size", String.valueOf(size));
            //获取物料成本
            PageDTO<ChuanyunPurCostDTO> pageInfo = clawFeign.findChuanyunPurCost(map);
            if (page <= pageInfo.getTotalPages()) {
                page += 1;
            } else {
                flag = false;
                break;
            }
            List<ChuanyunPurCostDTO> list = pageInfo.getContent();
            list.forEach(chuanyunPurCostDTO -> {
                try {
                    if (chuanyunPurCostDTO != null) {
                        PurCostDTO purCostDTO = new PurCostDTO(chuanyunPurCostDTO);
                        var purcostString = objectMapper.writeValueAsString(purCostDTO);
                        ChuanyunSaveDTO chuanyunSaveDTO;
                        //根据项目号判断是否存在
                        List<String> macthers = new ArrayList<>();
                        macthers.add("F0000001_2," + chuanyunPurCostDTO.getProjectcode());
                        var filter = Filter.instance(0, 1, true, "And", macthers);
                        var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunPurCostDO.SCHEMA_CODE, filter);
                        if (chuanyunFindAllResponse.getReturnData() == null) {
                            //新增
                            chuanyunSaveDTO = chuanYunManager.save(ChuanyunPurCostDO.SCHEMA_CODE, purcostString, true);
                        } else {
                            List<ChuanyunPurCostDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                            });
                            ChuanyunPurCostDO chuanyunPurCostDO = result.get(0);
                            //更新
                            chuanyunSaveDTO = chuanYunManager.update(ChuanyunPurCostDO.SCHEMA_CODE, chuanyunPurCostDO.getObjectId(), purcostString);
                        }
                        if (chuanyunSaveDTO.getSuccessful()) {
                            log.info("新增项目物料成本成功");
                        } else {
                            log.warn("新增项目物料成本失败");
                        }
                    }
                } catch (JsonProcessingException | BigSizeException e) {
                    e.printStackTrace();
                }
            });
        }
    }


    /**
     * 更新采购合同管理
     */
    private void savePurContract() {
        log.info("开始更新采购合同管理");
        var objectMapper = new ObjectMapper();
        var page = 1;
        var size = 20;
        var flag = true;
        while (flag) {
            Map<String, String> map = new HashMap<>(1);
            map.put("page", String.valueOf(page));
            map.put("size", String.valueOf(size));
            PageDTO<ChuanyunPurContractDTO> pageInfo = clawFeign.findChuanyunPurContract(map);
            if (page <= pageInfo.getTotalPages()) {
                page += 1;
            } else {
                flag = false;
                break;
            }
            List<ChuanyunPurContractDTO> list = pageInfo.getContent();
            list.forEach(chuanyunPurContractDTO -> {
                try {
                    if (!StringUtils.isEmpty(chuanyunPurContractDTO.getOrgname())) {
                        chuanyunPurContractDTO.setOrgname(chuanyunCompanyDao.findByCompanyName(chuanyunPurContractDTO.getOrgname()).getObjectId());
                    }
                    if (!StringUtils.isEmpty(chuanyunPurContractDTO.getOpername())) {
                        chuanyunPurContractDTO.setOpername(chuanyunUserDao.findByName(chuanyunPurContractDTO.getOpername()).getObjectId());
                    }
                    PurContractDTO purContractDTO = new PurContractDTO(chuanyunPurContractDTO);
                    purContractDTO.setDataSource("U9");
                    DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy/MM/dd HH:mm:ss");
                    String newDate = dateTimeFormatter.format(chuanyunPurContractDTO.getBusinessdate());
                    LocalDateTime dt = LocalDateTime.parse(newDate, dateTimeFormatter);
                    purContractDTO.setSignedDateTime(dt);
                    if (!StringUtils.isEmpty(chuanyunPurContractDTO.getProjectcode())) {
                        List<String> projectmatchers = new ArrayList<>();
                        projectmatchers.add("F0000002_2," + chuanyunPurContractDTO.getProjectcode());
                        var projectfilter = Filter.instance(0, Integer.MAX_VALUE, true, "And", projectmatchers);
                        ChuanyunFindAllBizDTO<Object> chuanyunFindResponse = null;
                        chuanyunFindResponse = chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE, projectfilter);
                        if (chuanyunFindResponse.getReturnData() != null) {
                            List<ChuanyunGroupProjectDO> projectresult = objectMapper.convertValue(chuanyunFindResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                            });
                            purContractDTO.setProjectId(projectresult.get(0).getObjectId());
                        }
                    }
                    List<ChuanyunPurContractOfferDTO> offerDTOS = clawFeign.findPurContractOfferByDocno(chuanyunPurContractDTO.getDocno());
                    List<PurContractOfferDTO> purContractOfferDTOS = new ArrayList<>();
                    offerDTOS.forEach(chuanyunPurContractOfferDTO -> {
                        if (!StringUtils.isEmpty(chuanyunPurContractOfferDTO.getSuppliername())) {
                            ChuanyunSupplierDO chuanyunSupplierDO = chuanyunSupplierDao.findBySupplierCode(chuanyunPurContractOfferDTO.getSuppliername());
                            if (chuanyunSupplierDO != null) {
                                chuanyunPurContractOfferDTO.setSuppliername(chuanyunSupplierDO.getObjectId());
                            }
                        }
                        PurContractOfferDTO purContractOfferDTO = new PurContractOfferDTO(chuanyunPurContractOfferDTO);
                        purContractOfferDTOS.add(purContractOfferDTO);
                    });
                    purContractDTO.setChuanyunPurContractOfferDOList(purContractOfferDTOS);
                    List<ChuanyunPurContractDetailDTO> detailDTOS = clawFeign.findPurContractDetailByDocno(chuanyunPurContractDTO.getDocno());
                    List<PurContractDetailDTO> purContractDetailDTOS = new ArrayList<>();
                    detailDTOS.forEach(chuanyunPurContractDetailDTO -> {
                        PurContractDetailDTO purContractDetailDTO = new PurContractDetailDTO(chuanyunPurContractDetailDTO);
                        purContractDetailDTOS.add(purContractDetailDTO);
                    });
                    purContractDTO.setChuanyunPurContractDetailDOList(purContractDetailDTOS);
                    var purcostString = objectMapper.writeValueAsString(purContractDTO);
                    List<String> macthers = new ArrayList<>();
                    macthers.add("F0000010_2," + chuanyunPurContractDTO.getDocno());
                    macthers.add("F0000016_2," + chuanyunPurContractDTO.getProjectcode());
                    var filter = Filter.instance(0, 1, true, "And", macthers);
                    var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunPurContractDO.SCHEMA_CODE, filter);
                    ChuanyunSaveDTO chuanyunSaveDTO;
                    if (chuanyunFindAllResponse.getReturnData() != null) {
                        List<ChuanyunPurContractDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                        });
                        chuanyunSaveDTO = chuanYunManager.delete(ChuanyunPurContractDO.SCHEMA_CODE, result.get(0).getObjectId());
                    }
                    chuanyunSaveDTO = chuanYunManager.save(ChuanyunPurContractDO.SCHEMA_CODE, purcostString, true);
                    if (chuanyunSaveDTO.getSuccessful()) {
                        log.info("新增采购合同管理成功");
                    } else {
                        log.warn("新增采购合同管理失败");
                    }

                } catch (JsonProcessingException | BigSizeException e) {
                    e.printStackTrace();
                }


            });
        }
    }

    /**
     * 更新销售合同管理
     */
    private void saveSalesContract() {
        log.info("开始更新销售合同管理");
        var objectMapper = new ObjectMapper();
        var page = 2;
        var size = 100;
        var flag = true;
        while (flag) {
            Map<String, String> map = new HashMap<>(1);
            map.put("page", String.valueOf(page));
            map.put("size", String.valueOf(size));
            PageDTO<ChuanyunSalesContractDTO> pageInfo = clawFeign.findChuanyunSalesContract(map);
            if (page <= pageInfo.getTotalPages()) {
                page += 1;
            } else {
                flag = false;
                break;
            }
            List<ChuanyunSalesContractDTO> list = pageInfo.getContent();
            list.forEach(chuanyunSalesContractDTO -> {
                try {
                    if (!StringUtils.isEmpty(chuanyunSalesContractDTO.getOrgname())) {
                        if (chuanyunCompanyDao.findByCompanyName(chuanyunSalesContractDTO.getOrgname()) != null) {
                            chuanyunSalesContractDTO.setOrgname(chuanyunCompanyDao.findByCompanyName(chuanyunSalesContractDTO.getOrgname()).getObjectId());
                        }
                    }
                    if (chuanyunClientDao.findByClientCode(chuanyunSalesContractDTO.getCustname()) != null) {
                        chuanyunSalesContractDTO.setCustname(chuanyunClientDao.findByClientCode(chuanyunSalesContractDTO.getCustname()).getObjectId());
                    }
                    if (!StringUtils.isEmpty(chuanyunSalesContractDTO.getCreatedby())) {
                        if (chuanyunUserDao.findByName(chuanyunSalesContractDTO.getCreatedby()) != null) {
                            chuanyunSalesContractDTO.setCreatedby(chuanyunUserDao.findByName(chuanyunSalesContractDTO.getCreatedby()).getObjectId());
                        }
                    }
                    SalesContractDTO salesContractDTO = new SalesContractDTO(chuanyunSalesContractDTO);
                    salesContractDTO.setContractStatus("已完结");
                    salesContractDTO.setDataSource("U9");
                    DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy/MM/dd HH:mm:ss");
                    String newDate = dateTimeFormatter.format(chuanyunSalesContractDTO.getBusinessdate());
                    LocalDateTime dt = LocalDateTime.parse(newDate, dateTimeFormatter);
                    salesContractDTO.setSignedDateTime(dt);
                    if (chuanyunSalesContractDTO.getReceivedate() != null) {
                        String receiveDate = dateTimeFormatter.format(chuanyunSalesContractDTO.getReceivedate());
                        LocalDateTime dt1 = LocalDateTime.parse(receiveDate, dateTimeFormatter);
                        salesContractDTO.setReceivedDateTime(dt1);
                    }
                    List<String> projectmatchers = new ArrayList<>();
                    projectmatchers.add("F0000002_2," + chuanyunSalesContractDTO.getProjectcode());
                    var projectfilter = Filter.instance(0, Integer.MAX_VALUE, true, "And", projectmatchers);
                    var chuanyunFindResponse = chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE, projectfilter);
                    if (chuanyunFindResponse.getReturnData() != null) {
                        List<ChuanyunGroupProjectDO> projectresult = objectMapper.convertValue(chuanyunFindResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                        });
                        salesContractDTO.setProjectId(projectresult.get(0).getObjectId());
                    }
                    List<ChuanyunSalesContractOfferDTO> offerDTOList = clawFeign.findAllByDocno(chuanyunSalesContractDTO.getDocno());
                    List<SalesContractOfferDTO> salesContractOfferDTOList = new ArrayList<>();
                    offerDTOList.forEach(chuanyunSalesContractOfferDTO -> {
                        SalesContractOfferDTO salesContractOfferDTO = new SalesContractOfferDTO(chuanyunSalesContractOfferDTO);
                        salesContractOfferDTOList.add(salesContractOfferDTO);
                    });
                    salesContractDTO.setChuanyunSaleContractDetailDOList(salesContractOfferDTOList);
                    var purcostString = objectMapper.writeValueAsString(salesContractDTO);
                    List<String> macthers = new ArrayList<>();
                    macthers.add("F0000001_2," + chuanyunSalesContractDTO.getDocno());
                    macthers.add("F0000025_2," + chuanyunSalesContractDTO.getProjectcode());
                    var filter = Filter.instance(0, Integer.MAX_VALUE, true, "And", macthers);
                    var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunSaleContractDO.SCHEMA_CODE, filter);
                    ChuanyunSaveDTO chuanyunSaveDTO;
                    if (chuanyunFindAllResponse.getReturnData() != null) {
                        List<ChuanyunSaleContractDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                        });
                        for (ChuanyunSaleContractDO chuanyunSaleContractDO : result) {
                            chuanyunSaveDTO = chuanYunManager.delete(ChuanyunSaleContractDO.SCHEMA_CODE, chuanyunSaleContractDO.getObjectId());
                        }
                    }
                    chuanyunSaveDTO = chuanYunManager.save(ChuanyunSaleContractDO.SCHEMA_CODE, purcostString, true);
                    if (chuanyunSaveDTO.getSuccessful()) {
                        log.info("新增销售合同管理成功");
                    } else {
                        log.warn("新增销售合同管理失败");
                        log.warn(chuanyunSaveDTO.getErrorMessage());
                        log.warn(purcostString);
                    }
                } catch (JsonProcessingException | BigSizeException e) {
                    e.printStackTrace();
                }
            });
        }
    }

    /**
     * 更新销售合同回款明细
     */
    private void saveSalesCollection() {
        log.info("开始更新销售合同回款明细");
        var objectMapper = new ObjectMapper();
        var start = 0;
        var totalCount = 0L;
        var flag = true;
        while (flag) {
            try {
                var filter = Filter.instance(start, start + pageSize, true);
                //获取销售合同管理
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunSaleContractDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                List<ChuanyunSaleContractDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                });
                totalCount = chuanyunFindAllResponse.getReturnData().getTotalCount();
                result.forEach(chuanyunSaleContractDO -> {
                    //根据合同号获取回款明细
                    List<ChuanyunSalesCollectionDetailDTO> collectionDetailDTOList = clawFeign.findCollectionByDocno(chuanyunSaleContractDO.getContractCode());
                    if (collectionDetailDTOList != null && collectionDetailDTOList.size() > 0) {
                        try {
                            SalesCollectionDTO salesCollectionDTO = new SalesCollectionDTO();
                            salesCollectionDTO.setDataSource("U9");
                            //销售合同id
                            salesCollectionDTO.setSaleContractId(chuanyunSaleContractDO.getObjectId());
                            //剩余回款金额
                            salesCollectionDTO.setSurplusAmount(chuanyunSaleContractDO.getTotalAmount());
                            //累计回款金额
                            BigDecimal amount = new BigDecimal(0);
                            salesCollectionDTO.setCumulativeAmount(amount);
                            //合同编号
                            salesCollectionDTO.setContractCode(chuanyunSaleContractDO.getContractCode());
                            //回款计划
                            Set<ReceivePlanDTO> receivePlanDTOS = new HashSet<>();
                            collectionDetailDTOList.forEach(chuanyunSalesCollectionDetailDTO -> {
                                ReceivePlanDTO receivePlanDTO = new ReceivePlanDTO(chuanyunSalesCollectionDetailDTO);
                                if (chuanyunSalesCollectionDetailDTO.getPlandate() != null) {
                                    DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy/MM/dd HH:mm:ss");
                                    String newDate = dateTimeFormatter.format(chuanyunSalesCollectionDetailDTO.getPlandate());
                                    LocalDateTime dt = LocalDateTime.parse(newDate, dateTimeFormatter);
                                    receivePlanDTO.setPlanDateTime(dt);
                                }
                                receivePlanDTOS.add(receivePlanDTO);
                            });
                            salesCollectionDTO.setPlanList(receivePlanDTOS);
                            var purcostString = objectMapper.writeValueAsString(salesCollectionDTO);
                            //根据合同号新增回款明细
                            List<String> macthers = new ArrayList<>();
                            macthers.add("F0000002_2," + chuanyunSaleContractDO.getContractCode());
                            var newfilter = Filter.instance(0, Integer.MAX_VALUE, true, "And", macthers);
                            var newchuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunPaymentReceiveDO.SCHEMA_CODE, newfilter);
                            ChuanyunSaveDTO chuanyunSaveDTO;
                            if (newchuanyunFindAllResponse.getReturnData() != null) {
                                List<ChuanyunPaymentReceiveDO> receiveDOSresult = objectMapper.convertValue(newchuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                                });
                                for (ChuanyunPaymentReceiveDO chuanyunPaymentReceiveDO : receiveDOSresult) {
                                    chuanyunSaveDTO = chuanYunManager.delete(ChuanyunPaymentReceiveDO.SCHEMA_CODE, chuanyunPaymentReceiveDO.getObjectId());
                                }
                            }
                            chuanyunSaveDTO = chuanYunManager.save(ChuanyunPaymentReceiveDO.SCHEMA_CODE, purcostString, true);
                            if (chuanyunSaveDTO.getSuccessful()) {
                                log.info("新增销售合同汇款明细成功");
                            } else {
                                log.warn("新增销售合同汇款明细失败");
                                log.warn(chuanyunSaveDTO.getErrorMessage());
                                log.warn(purcostString);
                            }
                        } catch (JsonProcessingException | BigSizeException e) {
                            e.printStackTrace();
                        }
                    }
                });
            } catch (Exception e) {
                e.printStackTrace();
                log.error(e.getMessage());
            }

        }
    }


    /**
     * 更新项目收付款
     */
    private void saveCashFlow() {
        log.info("开始更新项目收付款");
        var objectMapper = new ObjectMapper();
        var page = 1;
        var size = 20;
        var flag = true;
        while (flag) {
            Map<String, String> map = new HashMap<>(1);
            map.put("page", String.valueOf(page));
            map.put("size", String.valueOf(size));
            //获取收付款
            PageDTO<ChuanyunProjectCashFlowDTO> pageInfo = clawFeign.findChuanyunProjectCashFlow(map);
            if (page <= pageInfo.getTotalPages()) {
                page += 1;
            } else {
                flag = false;
                break;
            }
            List<ChuanyunProjectCashFlowDTO> list = pageInfo.getContent();
            list.forEach(chuanyunProjectCashFlowDTO -> {
                try {
                    if (chuanyunProjectCashFlowDTO != null) {
                        CashFlowDTO cashFlowDTO = new CashFlowDTO(chuanyunProjectCashFlowDTO);
                        var purcostString = objectMapper.writeValueAsString(cashFlowDTO);
                        ChuanyunSaveDTO chuanyunSaveDTO;
                        //根据项目号判断是否存在
                        List<String> macthers = new ArrayList<>();
                        macthers.add("F0000001_2," + chuanyunProjectCashFlowDTO.getProjectcode());
                        var filter = Filter.instance(0, 1, true, "And", macthers);
                        var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunProjectCashFlowDO.SCHEMA_CODE, filter);
                        if (chuanyunFindAllResponse.getReturnData() == null) {
                            //新增
                            chuanyunSaveDTO = chuanYunManager.save(ChuanyunProjectCashFlowDO.SCHEMA_CODE, purcostString, true);
                        } else {
                            List<ChuanyunProjectCashFlowDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                            });
                            ChuanyunProjectCashFlowDO chuanyunProjectCashFlowDO = result.get(0);
                            //更新
                            chuanyunSaveDTO = chuanYunManager.update(ChuanyunProjectCashFlowDO.SCHEMA_CODE, chuanyunProjectCashFlowDO.getObjectId(), purcostString);
                        }
                        if (chuanyunSaveDTO.getSuccessful()) {
//                            log.info("新增项目收付款成功");
                        } else {
                            log.warn("新增项目收付款失败");
                        }
                    }
                } catch (JsonProcessingException | BigSizeException e) {
                    e.printStackTrace();
                }
            });
        }
    }

    /**
     * U9供应商信息
     */
    private void saveSupplier() {
        log.info("开始更新供应商信息");
        var objectMapper = new ObjectMapper();
        var page = 1;
        var size = 20;
        var flag = true;
        while (flag) {
            Map<String, String> map = new HashMap<>(1);
            map.put("page", String.valueOf(page));
            map.put("size", String.valueOf(size));
            //获取供应商信息
            PageDTO<ChuanyunSupplierDTO> pageInfo = clawFeign.findChuanyunSupplier(map);
            if (page <= pageInfo.getTotalPages()) {
                page += 1;
            } else {
                flag = false;
                break;
            }
            List<ChuanyunSupplierDTO> list = pageInfo.getContent();
            list.forEach(chuanyunSupplierDTO -> {
                SupplierDTO supplierDTO = new SupplierDTO(chuanyunSupplierDTO);
                try {
                    var supplierString = objectMapper.writeValueAsString(supplierDTO);
                    ChuanyunSaveDTO chuanyunSaveDTO;
                    //根据编码判断供应商是否存在
                    List<String> macthers = new ArrayList<>();
                    macthers.add("F0000002_2," + chuanyunSupplierDTO.getCode());
                    var filter = Filter.instance(0, 1, true, "And", macthers);
                    var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunSupplierDO.SCHEMA_CODE, filter);
                    if (chuanyunFindAllResponse.getReturnData() == null) {
                        chuanyunSaveDTO = chuanYunManager.save(ChuanyunSupplierDO.SCHEMA_CODE, supplierString, true);
                    } else {
                        List<ChuanyunSupplierDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                        });
                        chuanyunSaveDTO = chuanYunManager.update(ChuanyunSupplierDO.SCHEMA_CODE, result.get(0).getObjectId(), supplierString);
                    }
                    if (chuanyunSaveDTO.getSuccessful()) {
                        log.info("新增供应商信息成功");
                    } else {
                        log.warn("新增供应商信息失败");
                    }
                } catch (JsonProcessingException | BigSizeException e) {
                    e.printStackTrace();
                }

            });
        }
    }

    /**
     * 新增人力成本
     */
    private void saveLaborCost() {
        log.info("开始更新项目人力成本");
        var objectMapper = new ObjectMapper();
        var page = 1;
        var size = 20;
        var flag = true;
        while (flag) {
            Map<String, String> map = new HashMap<>(1);
            map.put("page", String.valueOf(page));
            map.put("size", String.valueOf(size));
            //获取物料成本
            //@TODO这个方法找不到实现，代码可能未提交
            PageDTO<ChuanyunLaborCostDTO> pageInfo = clawFeign.findChuanyunLaborCost(map);
            if (page <= pageInfo.getTotalPages()) {
                page += 1;
            } else {
                flag = false;
                break;
            }
            List<ChuanyunLaborCostDTO> list = pageInfo.getContent();
            pageInfo.forEach(chuanyunLaborCostDTO -> {
                try {
                    if (chuanyunLaborCostDTO != null) {
                        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy/MM/dd HH:mm:ss");
                        String newDate = dateTimeFormatter.format(chuanyunLaborCostDTO.getChuanyunLaborCostPrimaryKey().getPeriod());
                        LocalDateTime dt = LocalDateTime.parse(newDate, dateTimeFormatter);
                        chuanyunLaborCostDTO.getChuanyunLaborCostPrimaryKey().setPeriod(dt);
                        String[] department = chuanyunLaborCostDTO.getChuanyunLaborCostPrimaryKey().getDepartment().split("/");

                        LaborCostDTO laborCostDTO = new LaborCostDTO(chuanyunLaborCostDTO);
                        if (department.length > 1 && (department[1].equals("EBG") || department[1].equals("IBG"))) {
                            laborCostDTO.setBg(department[1]);
                        }
                        laborCostDTO.setDataType("U9");
                        var purcostString = objectMapper.writeValueAsString(laborCostDTO);
                        log.info(purcostString);
                        ChuanyunSaveDTO chuanyunSaveDTO;
                        //根据项目号判断是否存在
                        List<String> matchers = new ArrayList<>();
                        matchers.add("F0000005_2," + chuanyunLaborCostDTO.getChuanyunLaborCostPrimaryKey().getProjectcode());
                        matchers.add("F0000002_2," + chuanyunLaborCostDTO.getChuanyunLaborCostPrimaryKey().getPeriod());
                        matchers.add("F0000018_2," + chuanyunLaborCostDTO.getChuanyunLaborCostPrimaryKey().getDepartment());
                        var filter = Filter.instance(0, 1, true, "And", matchers);
                        var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunMonthLaborCostDO.SCHEMA_CODE, filter);
                        if (chuanyunFindAllResponse.getReturnData() == null) {
                            //新增
                            chuanyunSaveDTO = chuanYunManager.save(ChuanyunMonthLaborCostDO.SCHEMA_CODE, purcostString, true);
                            log.info("人力成本新增成功");
                        } else {
                            List<ChuanyunMonthLaborCostDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                            });
                            if (result.size() > 0) {
                                ChuanyunMonthLaborCostDO chuanyunMonthLaborCostDO = result.get(0);
                                //更新
                                if (chuanyunMonthLaborCostDO.getAmount().compareTo(chuanyunLaborCostDTO.getCostamount()) != 0) {
                                    chuanyunSaveDTO = chuanYunManager.update(ChuanyunMonthLaborCostDO.SCHEMA_CODE, chuanyunMonthLaborCostDO.getObjectId(), purcostString);
                                    log.info("人力成本更新成功");
                                }
                            }
                        }
                    }
                } catch (JsonProcessingException | BigSizeException e) {
                    e.printStackTrace();
                }
            });
        }
    }

    /**
     * 新增人力成本
     */
    public void saveDbLaborCost() {
        chuanyunMonthLaborCostNewDao.deleteAllInBatch();
        log.info("开始更新项目人力成本");
        var objectMapper = new ObjectMapper();
        var page = 1;
        var size = 50;
        var flag = true;
        while (flag) {
            Map<String, String> map = new HashMap<>(1);
            map.put("page", String.valueOf(page));
            map.put("size", String.valueOf(size));
            //获取物料成本
            //@TODO这个方法找不到实现，代码可能未提交
            PageDTO<ChuanyunLaborCostDTO> pageInfo = clawFeign.findChuanyunLaborCost(map);
            if (page <= pageInfo.getTotalPages()) {
                page += 1;
            } else {
                flag = false;
                break;
            }
            List<ChuanyunLaborCostDTO> list = pageInfo.getContent();
            List<ChuanyunMonthLaborCostNewDO> saveDbList = new ArrayList<>();
            list.forEach(chuanyunLaborCostDTO -> {
                if (chuanyunLaborCostDTO != null) {
                    ChuanyunMonthLaborCostNewDO monthLaborCostDO = new ChuanyunMonthLaborCostNewDO();
                    monthLaborCostDO.setProjectCode(chuanyunLaborCostDTO.getChuanyunLaborCostPrimaryKey().getProjectcode());
                    monthLaborCostDO.setLogDate(chuanyunLaborCostDTO.getChuanyunLaborCostPrimaryKey().getPeriod());
                    monthLaborCostDO.setDepartment(chuanyunLaborCostDTO.getChuanyunLaborCostPrimaryKey().getDepartment());
                    monthLaborCostDO.setAmount(chuanyunLaborCostDTO.getCostamount());
                    monthLaborCostDO.setDataType("U9");
                    saveDbList.add(monthLaborCostDO);
                }
            });
            chuanyunMonthLaborCostNewDao.saveAll(saveDbList);
        }
        this.delChuanyunRepeatWorkHour();
    }

    private void delChuanyunRepeatWorkHour() {
        ObjectMapper objectMapper = new ObjectMapper();
        List<String> errorObjectIdList = new ArrayList<>();
        var start = 0;
        var pageSize = 200;
        var flag = true;
        while (flag) {
            try {
                //从氚云查询数据
                var filter = Filter.instance(start, start + pageSize, true);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunMonthLaborCostNewDTO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                //转化为POJO
                List<ChuanyunMonthLaborCostNewDTO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                });
                result.forEach(chuanyunMonthLaborCostDO -> {
                    List<ChuanyunMonthLaborCostNewDO> monthLaborCostNewDOList = chuanyunMonthLaborCostNewDao.findAllByProjectCodeAndLogDateAndDepartment(chuanyunMonthLaborCostDO.getProjectCode(), chuanyunMonthLaborCostDO.getLogDate(), chuanyunMonthLaborCostDO.getDepartment());
                    if (monthLaborCostNewDOList.isEmpty()) {
                        log.info("查询到异常的数据：{}", chuanyunMonthLaborCostDO);
                        errorObjectIdList.add(chuanyunMonthLaborCostDO.getObjectId());
                    }
                });
            } catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        errorObjectIdList.forEach(objectId -> chuanYunManager.delete(ChuanyunMonthLaborCostNewDTO.SCHEMA_CODE, objectId));
    }

    public void saveLaborCostWgTest() {
        saveDbLaborCostWg();
    }


    /**
     * 新增项目工作组人力成本
     */
    private void saveDbLaborCostWg() {
        chuanyunMonthLaborCostWgDao.deleteAllInBatch();
        log.info("开始保存项目工作组人力成本");
        var page = 1;
        var size = 50;
        var flag = true;
        while (flag) {
            Map<String, String> map = new HashMap<>(1);
            map.put("page", String.valueOf(page));
            map.put("size", String.valueOf(size));
            //获取物料成本
            PageInfo<ChuanyunLaborCostWgDTO> pageInfo = clawFeign.findChuanyunLaborCostWg(map);
            if (page <= pageInfo.getPages()) {
                page += 1;
            } else {
                flag = false;
                break;
            }
            List<ChuanyunLaborCostWgDTO> list = pageInfo.getList();

            ArrayList<ChuanyunMonthLaborCostWgDTO> laborCostWgDTOS = new ArrayList<>();

            list.forEach(laborCostWgDTO -> {
                if (!Objects.isNull(laborCostWgDTO)) {
                    // 保存至数据库
                    // 查询数据库比较数据
                    String projectCode = laborCostWgDTO.getProjectcode();
                    LocalDateTime time = laborCostWgDTO.getPeriod();
                    String department = laborCostWgDTO.getDepartment();
                    String workGroupId = laborCostWgDTO.getWorkgroupid();
                    // @fixme 避免默认工作组相同项目号相同时间相同部门数据相互影响，所以直接删除全表数据，重新全部保存。
                    ChuanyunMonthLaborCostWgDTO chuanyunMonthLaborCostWgDTO = toMonthLaborCostWg(laborCostWgDTO, projectCode, time, workGroupId, department);
                    laborCostWgDTOS.add(chuanyunMonthLaborCostWgDTO);

//                    List<ChuanyunMonthLaborCostWgDTO> laborCostWg = chuanyunMonthLaborCostWgDao.findAllByProjectCodeAndLogDateAndWorkGroupIdAndDepartment(projectCode, time, workGroupId, department);
//
//                    if (laborCostWg.isEmpty()) {
//                        //新增
//                        SaveMonthLaborCostWg(laborCostWgDTO, projectCode, time, workGroupId, department);
//                    } else {
//                        if (laborCostWg.size() > 1) {
//                            chuanyunMonthLaborCostWgDao.deleteAll(laborCostWg);
//                            //新增
//                            SaveMonthLaborCostWg(laborCostWgDTO, projectCode, time, workGroupId, department);
//                        } else if (laborCostWg.size() == 1) {
//                            Optional<ChuanyunMonthLaborCostWgDTO> first = laborCostWg.stream().findFirst();
//                            ChuanyunMonthLaborCostWgDTO chuanyunMonthLaborCostWgDTO = first.get();
//                            BigDecimal amount = chuanyunMonthLaborCostWgDTO.getAmount();
//                            BigDecimal costAmount = laborCostWgDTO.getCostamount();
//                            if (amount.compareTo(costAmount) != 0) {
//                                chuanyunMonthLaborCostWgDTO.setAmount(amount);
//                                chuanyunMonthLaborCostWgDao.save(chuanyunMonthLaborCostWgDTO);
//                                log.info("人力成本发生变化，更新人力成本: {}", chuanyunMonthLaborCostWgDTO);
//                            }
//                        }
//                    }
                }

            });
            chuanyunMonthLaborCostWgDao.saveAll(laborCostWgDTOS);
        }
        log.info("工作组人力成本保存至数据库完成");
    }

    private ChuanyunMonthLaborCostWgDTO toMonthLaborCostWg(ChuanyunLaborCostWgDTO laborCostWgDTO, String projectCode, LocalDateTime time, String workGroupId, String department) {
        ChuanyunMonthLaborCostWgDTO monthLaborCostWgDTO = new ChuanyunMonthLaborCostWgDTO();
        monthLaborCostWgDTO.setProjectCode(projectCode);
        monthLaborCostWgDTO.setProjectId(laborCostWgDTO.getProjectid());
        monthLaborCostWgDTO.setWorkGroupId(workGroupId);
        monthLaborCostWgDTO.setWorkGroupName(laborCostWgDTO.getWorkgroupname());
        monthLaborCostWgDTO.setAmount(laborCostWgDTO.getCostamount() == null ? new BigDecimal("0.00") : laborCostWgDTO.getCostamount());
        monthLaborCostWgDTO.setLogDate(time);
        monthLaborCostWgDTO.setDepartment(department);
        monthLaborCostWgDTO.setDataType("U9");
        return monthLaborCostWgDTO;

    }

    /**
     * 更新子项目人力成本
     */
    private void saveSubLaborCost() {
        log.info("开始更新子项目人力成本");
        var objectMapper = new ObjectMapper();
        var page = 1;
        var size = 20;
        var flag = true;
        while (flag) {
            Map<String, String> map = new HashMap<>(1);
            map.put("page", String.valueOf(page));
            map.put("size", String.valueOf(size));
            //获取物料成本
            PageDTO<ChuanyunSubLaborCostDTO> pageInfo = clawFeign.findChuanyunSubLaborCost(map);
            if (page <= pageInfo.getTotalPages()) {
                page += 1;
            } else {
                flag = false;
                break;
            }
            List<ChuanyunSubLaborCostDTO> list = pageInfo.getContent();
            list.forEach(chuanyunSubLaborCostDTO -> {
                try {
                    DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy/MM/dd HH:mm:ss");
                    String newDate = dateTimeFormatter.format(chuanyunSubLaborCostDTO.getChuanyunSubLaborCostPrimaryKey().getPeriod());
                    LocalDateTime dt = LocalDateTime.parse(newDate, dateTimeFormatter);
                    chuanyunSubLaborCostDTO.getChuanyunSubLaborCostPrimaryKey().setPeriod(dt);
                    String[] deparmentName = chuanyunSubLaborCostDTO.getChuanyunSubLaborCostPrimaryKey().getDepartment().split("/");
                    chuanyunSubLaborCostDTO.getChuanyunSubLaborCostPrimaryKey().setDepartment(deparmentName[deparmentName.length - 1]);
                    SubLaborCostDTO subLaborCostDTO = new SubLaborCostDTO(chuanyunSubLaborCostDTO);
                    var purcostString = objectMapper.writeValueAsString(subLaborCostDTO);
                    ChuanyunSaveDTO chuanyunSaveDTO;
                    List<String> matchers = new ArrayList<>();
                    matchers.add("F0000016_2," + chuanyunSubLaborCostDTO.getChuanyunSubLaborCostPrimaryKey().getSubprojectcode());
                    matchers.add("F0000002_2," + chuanyunSubLaborCostDTO.getChuanyunSubLaborCostPrimaryKey().getPeriod());
                    matchers.add("F0000017_2," + chuanyunSubLaborCostDTO.getChuanyunSubLaborCostPrimaryKey().getDepartment());
                    var filter = Filter.instance(0, 1, true, "And", matchers);
                    var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunSubLaborCostDO.SCHEMA_CODE, filter);
                    if (chuanyunFindAllResponse.getReturnData() != null) {
                        List<ChuanyunSubLaborCostDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                        });
                        chuanYunManager.delete(ChuanyunSubLaborCostDO.SCHEMA_CODE, result.get(0).getObjectId());
                    }
                    chuanyunSaveDTO = chuanYunManager.save(ChuanyunSubLaborCostDO.SCHEMA_CODE, purcostString, true);
                } catch (Exception e) {
                    e.printStackTrace();
                    log.info(e.getMessage());
                }
            });
        }
    }

    /**
     * 更新项目预决算统计
     */
    private void saveBudgetStatistics() {
        log.info("开始更新项目预决算");
        List<ChuanyunBudgetStatisticsDO> list = chuanyunBudgetStatisticsDao.findAll();
        list.forEach(chuanyunBudgetStatisticsDO -> {
            if (!StringUtils.isEmpty(chuanyunBudgetStatisticsDO.getProject())) {
                var objectMapper = new ObjectMapper();
                try {
                    //集团总表中查找项目
                    List<String> projectmatchers = new ArrayList<>();
                    projectmatchers.add("Name_2," + chuanyunBudgetStatisticsDO.getProject());
                    var projectfilter = Filter.instance(0, 1, true, "And", projectmatchers);
                    var chuanyunFindResponse = chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE, projectfilter);
                    if (chuanyunFindResponse.getReturnData() != null) {
                        List<ChuanyunGroupProjectDO> projectresult = objectMapper.convertValue(chuanyunFindResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                        });
                        chuanyunBudgetStatisticsDO.setProject(projectresult.get(0).getObjectId());
                        List<String> matchers = new ArrayList<>();
                        ChuanyunSaveDTO chuanyunSaveDTO;
                        matchers.add("F0000001_2," + chuanyunBudgetStatisticsDO.getProject());
                        matchers.add("F0000002_2," + chuanyunBudgetStatisticsDO.getFeeType());
                        var filter = Filter.instance(0, 1, true, "And", matchers);
                        var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunBudgetAmountDO.SCHEMA_CODE, filter);
                        BudgetDTO budgetDTO = new BudgetDTO(chuanyunBudgetStatisticsDO);
                        budgetDTO.setDate(LocalDateTime.of(2020, 2, 1, 0, 0));
                        var budgetString = objectMapper.writeValueAsString(budgetDTO);
                        if (chuanyunFindAllResponse.getReturnData() != null) {
                            List<ChuanyunBudgetAmountDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                            });
                            ChuanyunBudgetAmountDO chuanyunBudgetAmountDO = result.get(0);
                            chuanyunSaveDTO = chuanYunManager.update(ChuanyunBudgetAmountDO.SCHEMA_CODE, chuanyunBudgetAmountDO.getObjectId(), budgetString);
                        } else {
                            chuanyunSaveDTO = chuanYunManager.save(ChuanyunBudgetAmountDO.SCHEMA_CODE, budgetString, true);
                        }
                        if (chuanyunSaveDTO.getSuccessful()) {
//                            log.info("新增项目预决算统计成功");
                        } else {
                            log.warn("新增项目预决算统计失败");
                        }
                    }
                } catch (JsonProcessingException | BigSizeException e) {
                    e.printStackTrace();
                }
            }
        });
    }

    @Data
    class PurCostDTO {
        @JsonProperty("F0000001")
        private String projectcode;

        /**
         * 采购成本
         */
        @JsonProperty("F0000002")
        private BigDecimal purcost;

        /**
         * 领料成本
         */
        @JsonProperty("F0000003")
        private BigDecimal issuedcost;

        public PurCostDTO(ChuanyunPurCostDTO chuanyunPurCostDTO) {
            if (null != chuanyunPurCostDTO) {
                if (null != chuanyunPurCostDTO.getProjectcode()) {
                    this.projectcode = chuanyunPurCostDTO.getProjectcode();
                }
                if (null != chuanyunPurCostDTO.getPurcost()) {
                    this.purcost = chuanyunPurCostDTO.getPurcost();
                }
                if (null != chuanyunPurCostDTO.getIssuedcost()) {
                    this.issuedcost = chuanyunPurCostDTO.getIssuedcost();
                }
            }
        }
    }

    @Data
    class SupplierDTO {
        /**
         * 供应商名称
         */
        @JsonProperty("F0000001")
        private String supplierName;

        /**
         * U9代码
         */
        @JsonProperty("F0000002")
        private String supplierCode;

        public SupplierDTO(ChuanyunSupplierDTO chuanyunSupplierDTO) {
            if (null != chuanyunSupplierDTO) {
                if (null != chuanyunSupplierDTO.getCode()) {
                    this.supplierCode = chuanyunSupplierDTO.getCode();
                }
                if (null != chuanyunSupplierDTO.getName()) {
                    this.supplierName = chuanyunSupplierDTO.getName();
                }
            }
        }
    }

    @Data
    class SubLaborCostDTO {
        @JsonProperty("F0000005")
        private String projectCode;

        @JsonProperty("F0000016")
        private String subProjectCode;

        @JsonSerialize(using = ChuanyunLocalDateTimeSerializer.class)
        @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
        @JsonProperty("F0000002")
        private LocalDateTime logDate;

        @JsonProperty("F0000017")
        private String department;

        @JsonProperty("F0000003")
        private BigDecimal amount;

        @JsonProperty("F0000004")
        private String dataType;

        public SubLaborCostDTO(ChuanyunSubLaborCostDTO chuanyunSubLaborCostDO) {
            if (null != chuanyunSubLaborCostDO) {
                if (null != chuanyunSubLaborCostDO.getCostamount()) {
                    this.amount = chuanyunSubLaborCostDO.getCostamount();
                }
                if (null != chuanyunSubLaborCostDO.getChuanyunSubLaborCostPrimaryKey().getDepartment()) {
                    this.department = chuanyunSubLaborCostDO.getChuanyunSubLaborCostPrimaryKey().getDepartment();
                }
                if (null != chuanyunSubLaborCostDO.getChuanyunSubLaborCostPrimaryKey().getPeriod()) {
                    this.logDate = chuanyunSubLaborCostDO.getChuanyunSubLaborCostPrimaryKey().getPeriod();
                }
                if (null != chuanyunSubLaborCostDO.getProjectcode()) {
                    this.projectCode = chuanyunSubLaborCostDO.getProjectcode();
                }
                if (null != chuanyunSubLaborCostDO.getChuanyunSubLaborCostPrimaryKey().getSubprojectcode()) {
                    this.subProjectCode = chuanyunSubLaborCostDO.getChuanyunSubLaborCostPrimaryKey().getSubprojectcode();
                }
            }
        }
    }

    @Data
    class LaborCostDTO {

        @JsonProperty("F0000005")
        private String projectCode;

        @JsonProperty("F0000003")
        private BigDecimal amount;

        @JsonProperty("F0000002")
        @JsonSerialize(using = ChuanyunLocalDateTimeSerializer.class)
        @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
        private LocalDateTime logDate;

        @JsonProperty("F0000018")
        private String departmentCode;

        @JsonProperty("F0000017")
        private String bg;

        @JsonProperty("F0000004")
        private String dataType;

        public LaborCostDTO(ChuanyunLaborCostDTO chuanyunMonthLaborCostDTO) {
            if (null != chuanyunMonthLaborCostDTO) {
                if (null != chuanyunMonthLaborCostDTO.getCostamount()) {
                    this.amount = chuanyunMonthLaborCostDTO.getCostamount();
                }
                if (null != chuanyunMonthLaborCostDTO.getChuanyunLaborCostPrimaryKey().getProjectcode()) {
                    this.projectCode = chuanyunMonthLaborCostDTO.getChuanyunLaborCostPrimaryKey().getProjectcode();
                }
                if (null != chuanyunMonthLaborCostDTO.getChuanyunLaborCostPrimaryKey().getPeriod()) {
                    this.logDate = chuanyunMonthLaborCostDTO.getChuanyunLaborCostPrimaryKey().getPeriod();
                }
                if (null != chuanyunMonthLaborCostDTO.getChuanyunLaborCostPrimaryKey().getDepartment()) {
                    this.departmentCode = chuanyunMonthLaborCostDTO.getChuanyunLaborCostPrimaryKey().getDepartment();
                }
            }
        }
    }

    @Data
    class LaborCostWgDTO {

        @JsonProperty("F0000005")
        private String projectCode;

        @JsonProperty("F0000003")
        private BigDecimal amount;

        @JsonProperty("F0000002")
        @JsonSerialize(using = ChuanyunLocalDateTimeSerializer.class)
        @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
        private LocalDateTime logDate;

        @JsonProperty("F0000015")
        private String departmentCode;

        @JsonProperty("F0000018")
        private String departmentName;

        @JsonProperty("F0000017")
        private String bg;

        @JsonProperty("F0000004")
        private String dataType;

        @JsonProperty("F0000019")
        private String workGroupId;

        @JsonProperty("F0000020")
        private String workGroupName;

        public LaborCostWgDTO(ChuanyunLaborCostWgDTO chuanyunMonthLaborCostWgDTO) {
            if (null != chuanyunMonthLaborCostWgDTO) {
                if (null != chuanyunMonthLaborCostWgDTO.getCostamount()) {
                    this.amount = chuanyunMonthLaborCostWgDTO.getCostamount();
                }
                if (null != chuanyunMonthLaborCostWgDTO.getProjectcode()) {
                    this.projectCode = chuanyunMonthLaborCostWgDTO.getProjectcode();
                }
                if (null != chuanyunMonthLaborCostWgDTO.getPeriod()) {
                    this.logDate = chuanyunMonthLaborCostWgDTO.getPeriod();
                }
                if (null != chuanyunMonthLaborCostWgDTO.getDepartment()) {
                    this.departmentName = chuanyunMonthLaborCostWgDTO.getDepartment();
                }
                if (!Objects.isNull(chuanyunMonthLaborCostWgDTO.getWorkgroupid())) {
                    this.workGroupId = chuanyunMonthLaborCostWgDTO.getWorkgroupid();
                }

                if (!Objects.isNull(chuanyunMonthLaborCostWgDTO.getWorkgroupname())) {
                    this.workGroupName = chuanyunMonthLaborCostWgDTO.getWorkgroupname();
                }

            }
        }
    }

    @Data
    class BudgetDTO {

        /**
         * 所属项目
         */
        @JsonProperty("F0000001")
        private String project;

        /**
         * 费用类型
         */
        @JsonProperty("F0000002")
        private String feeType;

        /**
         * 预算金额
         */
        @JsonProperty("F0000003")
        private BigDecimal budgetAmount;

        /**
         * 默认日期（4月份）
         */
        @JsonProperty("F0000006")
        @JsonSerialize(using = ChuanyunLocalDateTimeSerializer.class)
        @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
        private LocalDateTime date;

        /**
         * 决算金额
         */
        @JsonProperty("F0000004")
        private BigDecimal finalAmount;

        /**
         * 预算绝算差异
         */
        @JsonProperty("F0000005")
        private BigDecimal differenceAmount;

        public BudgetDTO(ChuanyunBudgetStatisticsDO chuanyunBudgetStatisticsDO) {
            if (null != chuanyunBudgetStatisticsDO) {
                if (null != chuanyunBudgetStatisticsDO.getBudgetAmount()) {
                    this.budgetAmount = chuanyunBudgetStatisticsDO.getBudgetAmount();
                }
                if (null != chuanyunBudgetStatisticsDO.getFinalAmount()) {
                    this.finalAmount = chuanyunBudgetStatisticsDO.getFinalAmount();
                }
                if (null != chuanyunBudgetStatisticsDO.getFeeType()) {
                    this.feeType = chuanyunBudgetStatisticsDO.getFeeType();
                }
                if (null != chuanyunBudgetStatisticsDO.getProject()) {
                    this.project = chuanyunBudgetStatisticsDO.getProject();
                }
                if (null != chuanyunBudgetStatisticsDO.getDifferenceAmount()) {
                    this.differenceAmount = chuanyunBudgetStatisticsDO.getDifferenceAmount();
                }
            }
        }
    }

    @Data
    class CashFlowDTO {
        @JsonProperty("F0000001")
        private String projectcode;

        /**
         * 收款金额
         */
        @JsonProperty("F0000002")
        private BigDecimal income;

        /**
         * 付款金额
         */
        @JsonProperty("F0000003")
        private BigDecimal outcome;

        public CashFlowDTO(ChuanyunProjectCashFlowDTO chuanyunProjectCashFlowDTO) {
            if (null != chuanyunProjectCashFlowDTO) {
                if (null != chuanyunProjectCashFlowDTO.getProjectcode()) {
                    this.projectcode = chuanyunProjectCashFlowDTO.getProjectcode();
                }
                if (null != chuanyunProjectCashFlowDTO.getOutcome()) {
                    this.outcome = chuanyunProjectCashFlowDTO.getOutcome();
                }
                if (null != chuanyunProjectCashFlowDTO.getIncome()) {
                    this.income = chuanyunProjectCashFlowDTO.getIncome();
                }
            }
        }
    }

    @Data
    class PurContractDTO {

        @JsonProperty("F0000010")
        private String contractCode;

        @JsonProperty("F0000055")
        @JsonSerialize(using = ChuanyunLocalDateTimeSerializer.class)
        @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
        private LocalDateTime signedDateTime;

        @JsonProperty("F0000018")
        private String companyId;

        @JsonProperty("F0000013")
        private String contractCreator;

        @JsonProperty("F0000011")
        private String contractName;

        @JsonProperty("F0000015")
        private String projectId;

        @JsonProperty("F0000016")
        private String projectCode;

        @JsonProperty("F0000057")
        private String dataSource;

        @JsonProperty("D001789F42572596a43740a297245705dbf4b1b2")
        @OneToMany(cascade = CascadeType.ALL, orphanRemoval = true)
        private List<PurContractOfferDTO> chuanyunPurContractOfferDOList;

        @JsonProperty("D001789F07ac5fe52dc041b0b9e691ef8984093f")
        @OneToMany(cascade = CascadeType.ALL, orphanRemoval = true)
        private List<PurContractDetailDTO> chuanyunPurContractDetailDOList;

        public PurContractDTO(ChuanyunPurContractDTO chuanyunPurContractDTO) {
            if (null != chuanyunPurContractDTO) {
                if (null != chuanyunPurContractDTO.getDocno()) {
                    this.contractCode = chuanyunPurContractDTO.getDocno();
                }
                if (null != chuanyunPurContractDTO.getOrgname()) {
                    this.companyId = chuanyunPurContractDTO.getOrgname();
                }
                if (null != chuanyunPurContractDTO.getOpername()) {
                    this.contractCreator = chuanyunPurContractDTO.getOpername();
                }
                if (null != chuanyunPurContractDTO.getDocname()) {
                    this.contractName = chuanyunPurContractDTO.getDocname();
                }
                if (null != chuanyunPurContractDTO.getProjectcode()) {
                    this.projectCode = chuanyunPurContractDTO.getProjectcode();
                }
            }
        }
    }

    @Data
    class PurContractOfferDTO {

        @JsonProperty("supplier")
        private String supplierName;

        @JsonProperty("procurementEquipmentName")
        private String equipmentName;

        @JsonProperty("F0000049")
        private BigDecimal number;

        @JsonProperty("taxRate")
        private BigDecimal contractTax;

        @JsonProperty("totalPrice")
        private BigDecimal contractAmount;

        public PurContractOfferDTO(ChuanyunPurContractOfferDTO chuanyunPurContractOfferDTO) {
            if (null != chuanyunPurContractOfferDTO) {
                if (null != chuanyunPurContractOfferDTO.getItemname()) {
                    this.equipmentName = chuanyunPurContractOfferDTO.getItemname();
                }
                if (null != chuanyunPurContractOfferDTO.getSuppliername()) {
                    this.supplierName = chuanyunPurContractOfferDTO.getSuppliername();
                }
                if (null != chuanyunPurContractOfferDTO.getQty()) {
                    this.number = chuanyunPurContractOfferDTO.getQty();
                }
                if (null != chuanyunPurContractOfferDTO.getTaxrate()) {
                    this.contractTax = chuanyunPurContractOfferDTO.getTaxrate();
                }
                if (null != chuanyunPurContractOfferDTO.getMny()) {
                    this.contractAmount = chuanyunPurContractOfferDTO.getMny();
                }
            }
        }
    }

    @Data
    class PurContractDetailDTO {

        @JsonProperty("F0000030")
        private String purContractNode;

        @JsonProperty("F0000047")
        private BigDecimal purAmount;

        @JsonProperty("F0000046")
        private BigDecimal purTax;

        public PurContractDetailDTO(ChuanyunPurContractDetailDTO chuanyunPurContractDetailDTO) {
            if (null != chuanyunPurContractDetailDTO) {
                if (null != chuanyunPurContractDetailDTO.getDesnote()) {
                    this.purContractNode = chuanyunPurContractDetailDTO.getDesnote();
                }
                if (null != chuanyunPurContractDetailDTO.getPlanaccount()) {
                    this.purAmount = chuanyunPurContractDetailDTO.getPlanaccount();
                }
                if (null != chuanyunPurContractDetailDTO.getPlanpercent()) {
                    this.purTax = chuanyunPurContractDetailDTO.getPlanpercent();
                }
            }
        }
    }

    @Data
    class SalesContractDTO {
        @JsonProperty("F0000001")
        private String contractCode;

        @JsonProperty("F0000003")
        @JsonSerialize(using = ChuanyunLocalDateTimeSerializer.class)
        @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
        private LocalDateTime signedDateTime;

        @JsonProperty("F0000002")
        private String companyId;

        @JsonProperty("F0000072")
        private String contractCreator;

        @JsonProperty("F0000076")
        private String contractStatus;

        @JsonProperty("F0000062")
        private String salesManagerDepartment;

        @JsonProperty("F0000027")
        private String clientId;

        @JsonProperty("F0000020")
        private String projectId;

        @JsonProperty("F0000025")
        private String projectCode;

        @JsonProperty("F0000045")
        @JsonSerialize(using = ChuanyunLocalDateTimeSerializer.class)
        @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
        private LocalDateTime receivedDateTime;

        @JsonProperty("F0000004")
        private String contractName;

        @JsonProperty("F0000005")
        private String contractType;

        @JsonProperty("F0000028")
        private String location;

        @JsonProperty("F0000029")
        private String ownerId;

        @JsonProperty("F0000024")
        private String ownerIndustry;

        @JsonProperty("F0000056")
        private String businessSource;

        @JsonProperty("F0000058")
        private String businessType;

        @JsonProperty("F0000006")
        private String contractSubType;

        @JsonProperty("F0000067")
        private String currency;

        @JsonProperty("F0000047")
        private BigDecimal totalAmountWithoutTax;

        @JsonProperty("F0000021")
        private BigDecimal totalAmount;

        @JsonProperty("F0000077")
        private String dataSource;

        @JsonProperty("D001789Ffc834ddecbfe44b084795aa48e80bfcf")
        @OneToMany(cascade = CascadeType.ALL, orphanRemoval = true)
        private List<SalesContractOfferDTO> chuanyunSaleContractDetailDOList;

        public SalesContractDTO(ChuanyunSalesContractDTO chuanyunSalesContractDTO) {
            if (null != chuanyunSalesContractDTO) {
                if (null != chuanyunSalesContractDTO.getDocno()) {
                    this.contractCode = chuanyunSalesContractDTO.getDocno();
                }
                if (null != chuanyunSalesContractDTO.getOrgname()) {
                    this.companyId = chuanyunSalesContractDTO.getOrgname();
                }
                if (null != chuanyunSalesContractDTO.getCreatedby()) {
                    this.contractCreator = chuanyunSalesContractDTO.getCreatedby();
                }
                if (null != chuanyunSalesContractDTO.getDeptcode()) {
                    this.salesManagerDepartment = chuanyunSalesContractDTO.getDeptcode();
                }
                if (null != chuanyunSalesContractDTO.getCustname()) {
                    this.clientId = chuanyunSalesContractDTO.getCustname();
                }
                if (null != chuanyunSalesContractDTO.getProjectcode()) {
                    this.projectCode = chuanyunSalesContractDTO.getProjectcode();
                }
                if (null != chuanyunSalesContractDTO.getContractname()) {
                    this.contractName = chuanyunSalesContractDTO.getContractname();
                }
                if (null != chuanyunSalesContractDTO.getContracttype()) {
                    this.contractType = chuanyunSalesContractDTO.getContracttype();
                }
                if (null != chuanyunSalesContractDTO.getSignaddress()) {
                    this.location = chuanyunSalesContractDTO.getSignaddress();
                }
                if (null != chuanyunSalesContractDTO.getOwnercust()) {
                    this.ownerId = chuanyunSalesContractDTO.getOwnercust();
                }
                if (null != chuanyunSalesContractDTO.getOwnertrade()) {
                    this.ownerIndustry = chuanyunSalesContractDTO.getOwnertrade();
                }
                if (null != chuanyunSalesContractDTO.getBusinessfrom()) {
                    this.businessSource = chuanyunSalesContractDTO.getBusinessfrom();
                }
                if (null != chuanyunSalesContractDTO.getBusinesstype()) {
                    this.businessType = chuanyunSalesContractDTO.getBusinesstype();
                }
                if (null != chuanyunSalesContractDTO.getContractsubtype()) {
                    this.contractSubType = chuanyunSalesContractDTO.getContractsubtype();
                }
                if (null != chuanyunSalesContractDTO.getCurrencyname()) {
                    this.currency = chuanyunSalesContractDTO.getCurrencyname();
                }
                if (null != chuanyunSalesContractDTO.getNetmoney()) {
                    this.totalAmountWithoutTax = chuanyunSalesContractDTO.getNetmoney();
                }
                if (null != chuanyunSalesContractDTO.getTotalmoney()) {
                    this.totalAmount = chuanyunSalesContractDTO.getTotalmoney();
                }
            }
        }
    }

    @Data
    class SalesContractOfferDTO {

        @JsonProperty("F0000010")
        private String productName;

        @JsonProperty("F0000012")
        private BigDecimal number;

        @JsonProperty("F0000070")
        private BigDecimal unitPrice;

        @JsonProperty("F0000071")
        private BigDecimal unitPriceNoTax;

        @JsonProperty("F0000015")
        private BigDecimal rate;

        @JsonProperty("F0000050")
        private BigDecimal priceWithoutTax;

        @JsonProperty("F0000013")
        private BigDecimal price;

        public SalesContractOfferDTO(ChuanyunSalesContractOfferDTO chuanyunSalesContractOfferDTO) {
            if (null != chuanyunSalesContractOfferDTO) {
                if (null != chuanyunSalesContractOfferDTO.getItemname()) {
                    this.productName = chuanyunSalesContractOfferDTO.getItemname();
                }
                if (null != chuanyunSalesContractOfferDTO.getQty()) {
                    this.number = chuanyunSalesContractOfferDTO.getQty();
                }
                if (null != chuanyunSalesContractOfferDTO.getOrderprice()) {
                    this.unitPrice = chuanyunSalesContractOfferDTO.getOrderprice();
                }
                if (null != chuanyunSalesContractOfferDTO.getOrderpricenotax()) {
                    this.unitPriceNoTax = chuanyunSalesContractOfferDTO.getOrderpricenotax();
                }
                if (null != chuanyunSalesContractOfferDTO.getTaxrate()) {
                    this.rate = chuanyunSalesContractOfferDTO.getTaxrate();
                }
                if (null != chuanyunSalesContractOfferDTO.getNetmoney()) {
                    this.priceWithoutTax = chuanyunSalesContractOfferDTO.getNetmoney();
                }
                if (null != chuanyunSalesContractOfferDTO.getTotalmoney()) {
                    this.price = chuanyunSalesContractOfferDTO.getTotalmoney();
                }
            }
        }
    }

    @Data
    class SalesCollectionDTO {

        @JsonProperty("F0000001")
        private String saleContractId;

        @JsonProperty("F0000002")
        private String contractCode;

        @JsonProperty("F0000027")
        private BigDecimal surplusAmount;

        @JsonProperty("F0000012")
        private BigDecimal cumulativeAmount;

        @JsonProperty("F0000047")
        private String dataSource;

        @JsonProperty("D001789F2a010fe1818a4020b1ff5a33533f9e44")
        @OneToMany(cascade = CascadeType.ALL, orphanRemoval = true, fetch = FetchType.EAGER)
        private Set<ReceivePlanDTO> planList;
    }

    @Data
    class ReceivePlanDTO {

        @JsonProperty("F0000033")
        @JsonSerialize(using = ChuanyunLocalDateTimeSerializer.class)
        @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
        private LocalDateTime planDateTime;

        @JsonProperty("F0000036")
        private BigDecimal receiveAmount;

        @JsonProperty("F0000035")
        private BigDecimal rate;

        @JsonProperty("F0000034")
        private String step;

        @JsonProperty("F0000032")
        private String receiveCondition;

        public ReceivePlanDTO(ChuanyunSalesCollectionDetailDTO chuanyunSalesCollectionDetailDTO) {
            if (null != chuanyunSalesCollectionDetailDTO) {
                if (null != chuanyunSalesCollectionDetailDTO.getPlanaccount()) {
                    this.receiveAmount = chuanyunSalesCollectionDetailDTO.getPlanaccount();
                }
                if (null != chuanyunSalesCollectionDetailDTO.getPlanpercent()) {
                    this.rate = chuanyunSalesCollectionDetailDTO.getPlanpercent();
                }
                if (null != chuanyunSalesCollectionDetailDTO.getDesnote()) {
                    this.step = chuanyunSalesCollectionDetailDTO.getDesnote();
                }
                if (null != chuanyunSalesCollectionDetailDTO.getAccountnote()) {
                    this.receiveCondition = chuanyunSalesCollectionDetailDTO.getAccountnote();
                }
            }
        }
    }
}
